/***********************************************************************/
/*                                                                     */
/* Program: tree.c                                                     */
/*                                                                     */
/* The algorithm is taken from the book:                               */
/*                                                                     */
/* "Fractals for the Classroom" part one, Springer Verlag by           */
/* H-O. Peitgen, H. Juurgens and D. Saupe.                             */
/*                                                                     */
/* Program by: Stefan Spaennare, Lund, June 1995                       */
/* Email: stefans@astro.lu.se                                          */
/* Latest update: 1998-08-22                                           */
/*                                                                     */
/***********************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <graphics.h>


#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

double ran2(idum)
long *idum;
{
   long j;
   long k;
   static long idum2=123456789;
   static long iy=0; 
   static long iv[NTAB];
   double temp;

   if (*idum <= 0) {
      if (-(*idum) < 1) {
         *idum=1;
      }
      else {
         *idum=-(*idum);
      } /* if */
      idum2=(*idum);
      for (j=NTAB+7; j>=0; j--) {
         k=(*idum)/IQ1;
	 *idum=IA1*(*idum-k*IQ1)-k*IR1;
	 if (*idum < 0) {
	    *idum += IM1;
	 } /* if */
	 if (j < NTAB) {
	    iv[j]=*idum;
	 } /* if */
      } /* for j */
      iy=iv[0];
   } /* if */
   k=(*idum)/IQ1;
   *idum=IA1*(*idum-k*IQ1)-k*IR1;
   if (*idum < 0) {
      *idum += IM1;
   } /* if */
   k=idum2/IQ2;
   idum2=IA2*(idum2-k*IQ2)-k*IR2;
   if (idum2 < 0) {
      idum2 += IM2;
   } /* if */
   j=iy/NDIV;
   iy=iv[j]-idum2;
   iv[j] = *idum;
   if (iy < 1) {
      iy += IMM1;
   } /* if */
   temp=AM*(double)(iy);
   if (temp > RNMX) {
      return(RNMX);
   }
   else {
      return(temp);
   } /* if */

} /* ran2 */


int main(argc,argv)
int argc;
char *argv[];
{
   int gdriver = DETECT,gmode;

   double flops,ftime,sfac,itfac;
   double left,w,wl,e1,e2,e3,e4,f1,f2,f3,f4,x,y,xn,yn,r,e5,f5;

   long i,j,itmax,it,xsize,ysize,xi,yi,col,bgcol,seed;
   long idum;


clrscr();
printf("Input tree colour (0-15) (6): ");
scanf("%D",&col);
printf("Input background colour (0-15) (0): ");
scanf("%D",&bgcol);
clrscr();

initgraph(&gdriver,&gmode, "");

xsize=640;
ysize=480;

setcolor(bgcol);

for (i=0; i<xsize; i++) {
   line(i,0,i,ysize);
} /* for i */

left=30.0;
w=200.0;
wl=w+left;

e1=0.4431*w; e2=0.2511*w; e3=0.5976*w; e4=0.4884*w; e5=0.8562*w;
f1=0.2452*w; f2=0.5692*w; f3=0.0969*w; f4=0.5069*w; f5=0.2513*w;

x=e1;
y=0.0;

itfac=(double)(ysize)/1024.0;
itmax=(long)(itfac*itfac*0.5*1000000.0+0.5);

sfac=4.8*itfac;

seed=3;
idum=-abs(seed)-1;


for (i=1; i<=itmax; i++) {

    r=ran2(&idum);

    if (r < 0.2) {
      xn=0.195*x-0.488*y+e1;
       yn=0.344*x+0.443*y+f1;
    } 
    else if (r < 0.4) {
       xn=0.462*x+0.414*y+e2;
       yn=-0.252*x+0.361*y+f2;
    }
    else if (r < 0.6) {
       xn=-0.058*x-0.070*y+e3;
       yn=0.453*x-0.111*y+f3;
    }
    else if (r < 0.8) {
       xn=-0.035*x+0.070*y+e4;
       yn=-0.469*x-0.022*y+f4;
    }
    else {
       xn=-0.637*x+0.0*y+e5;
       yn=0.0*x+0.501*y+f5;
    } /* if */

    xi=(int)(sfac*(xn+left-25.0)+0.5)+80;
    yi=(int)(sfac*(wl-yn-20.0)+0.5);

    putpixel(xi,yi,col);

    x=xn;
    y=yn;

} /* for i */

getch();

closegraph();

} /* End */
